/*
  This uses z = a*sin(wl*PI*(t-sqrt(x*x+y*y))
*/
#include <stdio.h>
#include <math.h>

#if defined _WIN32 || defined _WIN64
#  include <windows.h>
#endif

#ifdef __APPLE__
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#endif

#include "glui.h"

/* Prototypes for the GLUT callback functions */
void display(void);
void reshape(int, int);
void slider1(float);
void slider2(float);
void slider3(float);
void slider4(float);
void keyboard(unsigned char, int, int);
void timer(int);

int window;
int windowHeight;   // this is set by the glut reshape() callback function
int windowWidth;    // this is set by the glut reshape() callback function

#define sliderInit 0.5
double angle1 = 0.0, angle2 = 0.0;
float lightX, lightY, lightZ;       /* rectangular coordinates of the light */
float centerX, centerY, centerZ;    /* rectangular coordinates of the lookat point */
int perspective = 1;                /* choose between perspective and parallel projections */
#define MAX_DIST 6

#define PI 3.14159265358979
double amp = 0.4;
double wl  = 1.0;
int lightOn = 1;
int wireFrameOn = 0;
int runAnimation = 1;
double deltaTime = 0.02;
double t = 0.0;  // t for "time"


int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(620, 640);
   glutInitWindowPosition(150, 150);
   window = glutCreateWindow("Hw 6 Demo");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutTimerFunc(20, timer, 0);   // timer() is called in 20 milliseconds (50 times a second)

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window,  1,  0, -1, 20, sliderInit, slider1);  // top edge
   gluiHorizontalSlider(window,  1, 20, -1, 20, sliderInit, slider2);
   gluiVerticalSlider  (window,  0, 40, 20, -1, sliderInit, slider3);  // left edge
   gluiVerticalSlider  (window, 20, 40, 20, -1, sliderInit, slider4);

   /* Initialize OpenGL state */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */

   glEnable(GL_DEPTH_TEST);

   glShadeModel (GL_SMOOTH);

   /* initialize global variables */
   lightZ = 10.0;
   centerX = 0;    /* rectangular coordinates of the lookat point */
   centerY = 0;
   centerZ = 0;

   printf("\nCS 45500 Homework Assignment 6 Demo:\n");
   printf("Light location is given in rectangular coordinates.\n");
   printf("   Top slider determines the x-coordinate of the light location.\n");
   printf("   Left slider determines the y-coordinate of the light location.\n");
   printf("The 'e' and 'E' keys display light location in rectangular coordinates.\n");
   printf("\n");
   printf("The \"spot light\" point is denoted in the scene by a small blue sphere.\n");
   printf("   The 'x' and 'X' keys change the \"spot light\" point's x-coordinate.\n");
   printf("   The 'y' and 'Y' keys change the \"spot light\" point's y-coordinate.\n");
   printf("   The 'z' and 'Z' keys change the \"spot light\" point's z-coordinate.\n");
   printf("The 'c' and 'C' keys display \"spot light\" point's coordinates.\n");
   printf("\n");
   printf("Use 'l' and 'L' keys to turn off and on the light.\n");
   printf("Use 'p' and 'P' keys to turn off and on the polygon fill.\n");
   printf("Use 's' and 'S' keys to stop and start the animation.\n");
   printf("Use 'f' and 'F' keys to make the animation faster and slower.\n");
   printf("Use 'a' and 'A' keys to increase and decrease the animation ampliture.\n");
   printf("Use 'w' and 'W' keys to increase and decrease the animation \"wavelength\".\n");
   printf("\n");
   printf("\n");

   glutMainLoop(); /* Enter event loop */
   return 0;       /* We never get here */
}//main()


void drawMesh()
{
   double x, y, z, nx, ny, length;
   int i, j;

   if (wireFrameOn)
   {// draw a wireframe
      glPolygonMode(GL_FRONT, GL_LINE);
      glPolygonMode(GL_BACK,  GL_LINE);
      glPushAttrib(GL_CURRENT_BIT);
      glColor3f(1.0, 0.0, 0.0);
   }

   glPushMatrix();
   glRotatef( angle1, 0.0, 1.0, 0.0);
   glRotatef( angle2, 1.0, 0.0, 0.0);
   for (i = 0; i < 20; i++)
   {
      glBegin(GL_QUAD_STRIP);
         for(j = 0; j <= 40; j++)
         {//z = a*sin(w*PI*(t-sqrt(x*x+y*y))
            x = -2.0+(j*0.1);
            y =  1.0-(i*0.1);
            z =  amp*sin(wl*PI*(t-sqrt(x*x + y*y)));
            nx = - amp*cos(wl*PI*(t-sqrt(x*x + y*y))) * wl * PI * x / sqrt(x*x + y*y);
            ny = - amp*cos(wl*PI*(t-sqrt(x*x + y*y))) * wl * PI * y / sqrt(x*x + y*y);
            length = sqrt(nx*nx + ny*ny + 1);
            glNormal3f(-nx/length, -ny/length, 1.0/length);
            glVertex3f(x, y, z);

            x = -2.0+(j*0.1);
            y =  1.0-(i*0.1)-0.1;
            z =  amp*sin(wl*PI*(t-sqrt(x*x + y*y)));
            nx = - amp*cos(wl*PI*(t-sqrt(x*x + y*y))) * wl * PI * x / sqrt(x*x + y*y);
            ny = - amp*cos(wl*PI*(t-sqrt(x*x + y*y))) * wl * PI * y / sqrt(x*x + y*y);
            length = sqrt(nx*nx + ny*ny + 1);
            glNormal3f(-nx/length, -ny/length, 1.0/length);
            glVertex3f(x, y, z);
         }
      glEnd();
   }
   glPopMatrix();

   glFlush();

   if (wireFrameOn)
   {
      glPolygonMode(GL_FRONT, GL_FILL);
      glPolygonMode(GL_BACK,  GL_FILL);
      glPopAttrib();
   }
}


/* The display() callback function */
void display( void )
{
   float left, right, bottom, top, zNear, zFar;

   /* Polished Gold material properties */
   GLfloat mat_f_ambient[]   = { 0.24725,  0.2245,   0.0645,   1.0 };
   GLfloat mat_f_diffuse[]   = { 0.34615,  0.3143,   0.0903,   1.0 };
   GLfloat mat_f_specular[]  = { 0.797357, 0.723991, 0.208006, 1.0 };
   GLfloat mat_f_shininess[] = { 83.2 };
   /* Jade material properties */
   GLfloat mat_b_ambient[]   = { 0.135,    0.2225,   0.1575,   0.95 };
   GLfloat mat_b_diffuse[]   = { 0.54,     0.89,     0.63,     0.95 };
   GLfloat mat_b_specular[]  = { 0.316228, 0.316228, 0.316228, 0.95 };
   GLfloat mat_b_shininess[] = { 12.8 };

   if (lightOn)
   {
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);

      /* light properties */
      GLfloat light_ambient[]  = { 0.0, 0.0, 0.0, 0.0 };
      GLfloat light_diffuse[]  = { 0.5, 0.5, 0.5, 0.0 }; // white light
      GLfloat light_specular[] = { 0.5, 0.5, 0.5, 0.0 };
      GLfloat global_ambient[] = { 0.1, 0.1, 0.1, 0.0 };

      GLfloat light_position[] = { lightX, lightY, lightZ, 0.0 };

      /* Set the properties of Light 0 */
      glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
      glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
      glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
      glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION,  1.0);
      glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION,    1.0);
      glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 1.0);

      /* Set global light properties */
      glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
      glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient ); // white ambient light

      /* Set the material properties */
      glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_f_ambient);
      glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_f_diffuse);
      glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_f_specular);
      glMaterialfv(GL_FRONT, GL_SHININESS, mat_f_shininess);
      glMaterialfv(GL_BACK, GL_AMBIENT,   mat_b_ambient);
      glMaterialfv(GL_BACK, GL_DIFFUSE,   mat_b_diffuse);
      glMaterialfv(GL_BACK, GL_SPECULAR,  mat_b_specular);
      glMaterialfv(GL_BACK, GL_SHININESS, mat_b_shininess);

      glLightfv(GL_LIGHT0, GL_POSITION, light_position);
   }
   else
   {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
   }

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  // clear the window

   // specify the viewport
   glViewport(40, 0, windowWidth-40, windowHeight-40); // left, bottom, width, height (in pixels)

   // set up viewing
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // parameters for glFrustum() or glOrtho()
   left = -1.0;
   right = 1.0;
   bottom = -1.0;
   top = 1.0;
   zNear = 2.0;
   zFar = 6.0;
   if (perspective)
   {
      glFrustum(left, right, bottom, top, zNear, zFar);
   }
   else /* use parallel projection */
   {
      glOrtho  (left, right, bottom, top, zNear, zFar);
   }

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   // eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ
   gluLookAt(0, 0, 4, 0, 0, 0, 0, 1, 0);

   drawMesh();

/*
   // draw three "coordinate" axes.
   glBegin(GL_LINES);
      glColor3f(1.0, 0.0, 0.0); // draw in red
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(5.0, 0.0, 0.0);
       glColor3f(0.0, 1.0, 0.0); // draw in green
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 5.0, 0.0);
       glColor3f(0.0, 0.0, 1.0); // draw in blue
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 5.0);
   glEnd();
*/

   /* draw something to mark the "lookat" point */
   glPushAttrib(GL_CURRENT_BIT);
     glColor3f(0.0, 0.0, 1.0);
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glPushMatrix();
       glTranslatef(centerX, centerY, centerZ);
       glutWireSphere(0.05, 8, 8);
     glPopMatrix();
   glPopAttrib();

   glutSwapBuffers();  // glui needs double buffering!
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback functions */
void slider1(float value)
{
   lightX = MAX_DIST*(value - 0.5);
   //printf("lightX = %f, lightY = %f, lightZ = %f\n", lightX, lightY, lightZ);
   glutPostRedisplay();
}//slider1()


void slider2(float value)
{
   angle1 = 360*(value - 0.5);

   glutPostRedisplay();
}//slider2()


void slider3(float value)
{
   lightY = MAX_DIST*(value - 0.5);
   //printf("lightX = %f, lightY = %f, lightZ = %f\n", lightX, lightY, lightZ);
   glutPostRedisplay();
}//slider3()


void slider4(float value)
{
   angle2 = 360*(value - 0.5);

   glutPostRedisplay();
}//slider4()


/* glut keyboard() call back function */
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 'x':
         centerX += 0.05;
         break;
      case 'X':
         centerX -= 0.05;
         break;
      case 'y':
         centerY += 0.05;
         break;
      case 'Y':
         centerY -= 0.05;
         break;
      case 'z':
         centerZ += 0.05;
         break;
      case 'Z':
         centerZ -= 0.05;
         break;
      case 'c':
      case 'C':
         printf("centerX = %f, centerY = %f, centerZ = %f\n", centerX, centerY, centerZ);
         break;

      case 'e':
      case 'E':
         printf("lightX = %f,  lightY = %f,  lightZ = %f\n",  lightX, lightY, lightZ);
         break;
      case 'p':
         wireFrameOn = 0;
         printf("Fill in polygons.\n");
         break;
      case 'P':
         wireFrameOn = 1;
         printf("Draw wireframe.\n");
         break;

      case 'l':
         lightOn = 0;
         break;
      case 'L':
         lightOn = 1;
         break;
      case 's':
         runAnimation = 0;
         break;
      case 'S':
         runAnimation = 1;
         break;
      case 'f':
         deltaTime += 0.01;
         printf("The \"time delta\" = %f\n", deltaTime);
         break;
      case 'F':
         deltaTime -= 0.01;
         printf("The \"time delta\" = %f\n", deltaTime);
         break;
      case 'a':
         amp += 0.01;
         printf("The amplitude = %f\n", amp);
         break;
      case 'A':
         amp -= 0.01;
         printf("The amplitude = %f\n", amp);
         break;
      case 'w':
         wl += 0.05;
         printf("The \"wave length\" = %f\n", wl);
         break;
      case 'W':
         wl -= 0.05;
         printf("The \"wave length\" = %f\n", wl);
         break;
      case 27:
         exit(0);
         break;

      default:
         break;
   }
   glutPostRedisplay();
}//keyboard()


/* glut timer() callback function */
void timer(int value)
{
   if (runAnimation)
   {
      t += deltaTime;
      //if ( t > 2.0*PI ) t = t - 2.0*PI;
   }
   glutTimerFunc(20, timer, 0); // timer() is called in 20 milliseconds (50 times a second)

   glutSetWindow(window);
   glutPostRedisplay();
}//timer()